"""Implementation of statechart statechart.
Generated by YAKINDU Statechart Tools code generator.
"""

import queue
import sys, os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src')))
from yakindu.rx import Observable

class Statechart:
	"""Implementation of the state machine Statechart.
	"""

	class State:
		""" State Enum
		"""
		(
			main_region_green,
			main_region_red,
			main_region_yellow,
			null_state
		) = range(4)
	
	
	def __init__(self):
		""" Declares all necessary variables including list of states, histories etc. 
		"""
		
		self.display_red = None
		self.display_red_observable = Observable()
		self.display_yellow = None
		self.display_yellow_observable = Observable()
		self.display_green = None
		self.display_green_observable = Observable()
		self.display_none = None
		self.display_none_observable = Observable()
		self.display_red_and_yellow = None
		self.display_red_and_yellow_observable = Observable()
		self.interrupt = None
		self.toggle_on_off = None
		self.set_timer_value = None
		self.set_timer_value_value = None
		self.set_timer_value_observable = Observable()
		
		self.in_event_queue = queue.Queue()
		
		# enumeration of all states:
		self.__State = Statechart.State
		self.__state_conf_vector_changed = None
		self.__state_vector = [None] * 1
		for __state_index in range(1):
			self.__state_vector[__state_index] = self.State.null_state
		
		# for timed statechart:
		self.timer_service = None
		self.__time_events = [None] * 3
		
		# initializations:
		self.__is_executing = False
	
	def is_active(self):
		"""Checks if the state machine is active.
		"""
		return self.__state_vector[0] is not self.__State.null_state
	
	def is_final(self):
		"""Checks if the statemachine is final.
		Always returns 'false' since this state machine can never become final.
		"""
		return False
			
	def is_state_active(self, state):
		"""Checks if the state is currently active.
		"""
		s = state
		if s == self.__State.main_region_green:
			return self.__state_vector[0] == self.__State.main_region_green
		if s == self.__State.main_region_red:
			return self.__state_vector[0] == self.__State.main_region_red
		if s == self.__State.main_region_yellow:
			return self.__state_vector[0] == self.__State.main_region_yellow
		return False
		
	def time_elapsed(self, event_id):
		"""Add time events to in event queue
		"""
		if event_id in range(3):
			self.in_event_queue.put(lambda: self.raise_time_event(event_id))
			self.run_cycle()
	
	def raise_time_event(self, event_id):
		"""Raise timed events using the event_id.
		"""
		self.__time_events[event_id] = True
	
	def __execute_queued_event(self, func):
		func()
	
	def __get_next_event(self):
		if not self.in_event_queue.empty():
			return self.in_event_queue.get()
		return None
	
	
	def raise_interrupt(self):
		"""Raise method for event interrupt.
		"""
		self.in_event_queue.put(self.__raise_interrupt_call)
		self.run_cycle()
	
	def __raise_interrupt_call(self):
		"""Raise callback for event interrupt.
		"""
		self.interrupt = True
	
	def raise_toggle_on_off(self):
		"""Raise method for event toggle_on_off.
		"""
		self.in_event_queue.put(self.__raise_toggle_on_off_call)
		self.run_cycle()
	
	def __raise_toggle_on_off_call(self):
		"""Raise callback for event toggle_on_off.
		"""
		self.toggle_on_off = True
	
	def __entry_action_main_region_green(self):
		"""Entry action for state 'Green'..
		"""
		self.timer_service.set_timer(self, 0, 1500, False)
		self.display_green_observable.next()
		
	def __entry_action_main_region_red(self):
		"""Entry action for state 'Red'..
		"""
		self.timer_service.set_timer(self, 1, (2 * 1000), False)
		self.display_red_observable.next()
		
	def __entry_action_main_region_yellow(self):
		"""Entry action for state 'Yellow'..
		"""
		self.timer_service.set_timer(self, 2, 500, False)
		self.display_yellow_observable.next()
		
	def __exit_action_main_region_green(self):
		"""Exit action for state 'Green'..
		"""
		self.timer_service.unset_timer(self, 0)
		
	def __exit_action_main_region_red(self):
		"""Exit action for state 'Red'..
		"""
		self.timer_service.unset_timer(self, 1)
		
	def __exit_action_main_region_yellow(self):
		"""Exit action for state 'Yellow'..
		"""
		self.timer_service.unset_timer(self, 2)
		
	def __enter_sequence_main_region_green_default(self):
		"""'default' enter sequence for state Green.
		"""
		self.__entry_action_main_region_green()
		self.__state_vector[0] = self.State.main_region_green
		self.__state_conf_vector_changed = True
		
	def __enter_sequence_main_region_red_default(self):
		"""'default' enter sequence for state Red.
		"""
		self.__entry_action_main_region_red()
		self.__state_vector[0] = self.State.main_region_red
		self.__state_conf_vector_changed = True
		
	def __enter_sequence_main_region_yellow_default(self):
		"""'default' enter sequence for state Yellow.
		"""
		self.__entry_action_main_region_yellow()
		self.__state_vector[0] = self.State.main_region_yellow
		self.__state_conf_vector_changed = True
		
	def __enter_sequence_main_region_default(self):
		"""'default' enter sequence for region main region.
		"""
		self.__react_main_region__entry_default()
		
	def __exit_sequence_main_region_green(self):
		"""Default exit sequence for state Green.
		"""
		self.__state_vector[0] = self.State.null_state
		self.__exit_action_main_region_green()
		
	def __exit_sequence_main_region_red(self):
		"""Default exit sequence for state Red.
		"""
		self.__state_vector[0] = self.State.null_state
		self.__exit_action_main_region_red()
		
	def __exit_sequence_main_region_yellow(self):
		"""Default exit sequence for state Yellow.
		"""
		self.__state_vector[0] = self.State.null_state
		self.__exit_action_main_region_yellow()
		
	def __exit_sequence_main_region(self):
		"""Default exit sequence for region main region.
		"""
		state = self.__state_vector[0]
		if state == self.State.main_region_green:
			self.__exit_sequence_main_region_green()
		elif state == self.State.main_region_red:
			self.__exit_sequence_main_region_red()
		elif state == self.State.main_region_yellow:
			self.__exit_sequence_main_region_yellow()
		
	def __react_main_region__entry_default(self):
		"""Default react sequence for initial entry .
		"""
		self.__enter_sequence_main_region_red_default()
		
	def __react(self, transitioned_before):
		"""Implementation of __react function.
		"""
		return transitioned_before
	
	
	def __main_region_green_react(self, transitioned_before):
		"""Implementation of __main_region_green_react function.
		"""
		transitioned_after = transitioned_before
		transitioned_after = self.__react(transitioned_before)
		if transitioned_after < 0:
			if self.__time_events[0]:
				self.__exit_sequence_main_region_green()
				self.__time_events[0] = False
				self.__enter_sequence_main_region_yellow_default()
				transitioned_after = 0
		return transitioned_after
	
	
	def __main_region_red_react(self, transitioned_before):
		"""Implementation of __main_region_red_react function.
		"""
		transitioned_after = transitioned_before
		transitioned_after = self.__react(transitioned_before)
		if transitioned_after < 0:
			if self.__time_events[1]:
				self.__exit_sequence_main_region_red()
				self.__time_events[1] = False
				self.__enter_sequence_main_region_green_default()
				transitioned_after = 0
		return transitioned_after
	
	
	def __main_region_yellow_react(self, transitioned_before):
		"""Implementation of __main_region_yellow_react function.
		"""
		transitioned_after = transitioned_before
		transitioned_after = self.__react(transitioned_before)
		if transitioned_after < 0:
			if self.__time_events[2]:
				self.__exit_sequence_main_region_yellow()
				self.__time_events[2] = False
				self.__enter_sequence_main_region_red_default()
				transitioned_after = 0
		return transitioned_after
	
	
	def __clear_in_events(self):
		"""Implementation of __clear_in_events function.
		"""
		self.interrupt = False
		self.toggle_on_off = False
		self.__time_events[0] = False
		self.__time_events[1] = False
		self.__time_events[2] = False
	
	
	def __micro_step(self):
		"""Implementation of __micro_step function.
		"""
		state = self.__state_vector[0]
		if state == self.State.main_region_green:
			self.__main_region_green_react(-1)
		elif state == self.State.main_region_red:
			self.__main_region_red_react(-1)
		elif state == self.State.main_region_yellow:
			self.__main_region_yellow_react(-1)
	
	
	def run_cycle(self):
		"""Implementation of run_cycle function.
		"""
		if self.timer_service is None:
			raise ValueError('Timer service must be set.')
		
		if self.__is_executing:
			return
		self.__is_executing = True
		next_event = self.__get_next_event()
		if next_event is not None:
			self.__execute_queued_event(next_event)
		condition_0 = True
		while condition_0:
			self.__micro_step()
			self.__clear_in_events()
			next_event = self.__get_next_event()
			if next_event is not None:
				self.__execute_queued_event(next_event)
			condition_0 = self.interrupt or self.toggle_on_off or self.__time_events[0] or self.__time_events[1] or self.__time_events[2]
		self.__is_executing = False
	
	
	def enter(self):
		"""Implementation of enter function.
		"""
		if self.timer_service is None:
			raise ValueError('Timer service must be set.')
		
		if self.__is_executing:
			return
		self.__is_executing = True
		self.__enter_sequence_main_region_default()
		self.__is_executing = False
	
	
	def exit(self):
		"""Implementation of exit function.
		"""
		if self.__is_executing:
			return
		self.__is_executing = True
		self.__exit_sequence_main_region()
		self.__is_executing = False
	
	
	def trigger_without_event(self):
		"""Implementation of triggerWithoutEvent function.
		"""
		self.run_cycle()
	
